@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0
-/* Copyright (C) 2011-2019  B.A.T.M.A.N. contributors:
+/* Copyright (C) 2011-2020  B.A.T.M.A.N. contributors:
  *
  * Linus Lüssing, Marek Lindner
  */
@@ -18,15 +18,14 @@
 #include <linux/jiffies.h>
 #include <linux/kernel.h>
 #include <linux/kref.h>
-#include <linux/list.h>
 #include <linux/netdevice.h>
 #include <linux/nl80211.h>
+#include <linux/prandom.h>
 #include <linux/random.h>
 #include <linux/rculist.h>
 #include <linux/rcupdate.h>
 #include <linux/rtnetlink.h>
 #include <linux/skbuff.h>
-#include <linux/slab.h>
 #include <linux/stddef.h>
 #include <linux/string.h>
 #include <linux/types.h>
@@ -42,18 +41,6 @@
 #include "routing.h"
 #include "send.h"
 
-/**
- * struct batadv_v_metric_queue_entry - list of hardif neighbors which require
- *  and metric update
- */
-struct batadv_v_metric_queue_entry {
-	/** @hardif_neigh: hardif neighbor scheduled for metric update */
-	struct batadv_hardif_neigh_node *hardif_neigh;
-
-	/** @list: list node for metric_queue */
-	struct list_head list;
-};
-
 /**
  * batadv_v_elp_start_timer() - restart timer for ELP periodic work
  * @hard_iface: the interface for which the timer has to be reset
@@ -63,7 +50,7 @@ static void batadv_v_elp_start_timer(struct batadv_hard_iface *hard_iface)
 	unsigned int msecs;
 
 	msecs = atomic_read(&hard_iface->bat_v.elp_interval) - BATADV_JITTER;
-	msecs += prandom_u32() % (2 * BATADV_JITTER);
+	msecs += prandom_u32_max(2 * BATADV_JITTER);
 
 	queue_delayed_work(batadv_event_workqueue, &hard_iface->bat_v.elp_wq,
 			   msecs_to_jiffies(msecs));
@@ -72,36 +59,25 @@ static void batadv_v_elp_start_timer(struct batadv_hard_iface *hard_iface)
 /**
  * batadv_v_elp_get_throughput() - get the throughput towards a neighbour
  * @neigh: the neighbour for which the throughput has to be obtained
- * @pthroughput: calculated throughput towards the given neighbour in multiples
- *  of 100kpbs (a value of '1' equals 0.1Mbps, '10' equals 1Mbps, etc).
  *
- * Return: true when value behind @pthroughput was set
+ * Return: The throughput towards the given neighbour in multiples of 100kpbs
+ *         (a value of '1' equals 0.1Mbps, '10' equals 1Mbps, etc).
  */
-static bool batadv_v_elp_get_throughput(struct batadv_hardif_neigh_node *neigh,
-					u32 *pthroughput)
+static u32 batadv_v_elp_get_throughput(struct batadv_hardif_neigh_node *neigh)
 {
 	struct batadv_hard_iface *hard_iface = neigh->if_incoming;
-	struct net_device *soft_iface = hard_iface->soft_iface;
 	struct ethtool_link_ksettings link_settings;
 	struct net_device *real_netdev;
 	struct station_info sinfo;
 	u32 throughput;
 	int ret;
 
-	/* don't query throughput when no longer associated with any
-	 * batman-adv interface
-	 */
-	if (!soft_iface)
-		return false;
-
 	/* if the user specified a customised value for this interface, then
 	 * return it directly
 	 */
 	throughput =  atomic_read(&hard_iface->bat_v.throughput_override);
-	if (throughput != 0) {
-		*pthroughput = throughput;
-		return true;
-	}
+	if (throughput != 0)
+		return throughput;
 
 	/* if this is a wireless device, then ask its throughput through
 	 * cfg80211 API
@@ -128,31 +104,28 @@ static bool batadv_v_elp_get_throughput(struct batadv_hardif_neigh_node *neigh,
 			 * possible to delete this neighbor. For now set
 			 * the throughput metric to 0.
 			 */
-			*pthroughput = 0;
-			return true;
+			return 0;
 		}
 		if (ret)
 			goto default_throughput;
-		if (!(sinfo.filled & BIT(NL80211_STA_INFO_EXPECTED_THROUGHPUT)))
-			goto default_throughput;
 
-		*pthroughput = sinfo.expected_throughput / 100;
-		return true;
+		if (sinfo.filled & BIT(NL80211_STA_INFO_EXPECTED_THROUGHPUT))
+			return sinfo.expected_throughput / 100;
+
+		/* try to estimate the expected throughput based on reported tx
+		 * rates
+		 */
+		if (sinfo.filled & BIT(NL80211_STA_INFO_TX_BITRATE))
+			return cfg80211_calculate_bitrate(&sinfo.txrate) / 3;
+
+		goto default_throughput;
 	}
 
-	memset(&link_settings, 0, sizeof(link_settings));
-
-	/* only use rtnl_trylock because the elp worker will be cancelled while
-	 * the rntl_lock is held. the cancel_delayed_work_sync() would otherwise
-	 * wait forever when the elp work_item was started and it is then also
-	 * trying to rtnl_lock
-	 */
-	if (!rtnl_trylock())
-		return false;
-
 	/* if not a wifi interface, check if this device provides data via
 	 * ethtool (e.g. an Ethernet adapter)
 	 */
+	memset(&link_settings, 0, sizeof(link_settings));
+	rtnl_lock();
 	ret = __ethtool_get_link_ksettings(hard_iface->net_dev, &link_settings);
 	rtnl_unlock();
 	if (ret == 0) {
@@ -163,15 +136,13 @@ static bool batadv_v_elp_get_throughput(struct batadv_hardif_neigh_node *neigh,
 			hard_iface->bat_v.flags &= ~BATADV_FULL_DUPLEX;
 
 		throughput = link_settings.base.speed;
-		if (throughput && throughput != SPEED_UNKNOWN) {
-			*pthroughput = throughput * 10;
-			return true;
-		}
+		if (throughput && throughput != SPEED_UNKNOWN)
+			return throughput * 10;
 	}
 
 default_throughput:
 	if (!(hard_iface->bat_v.flags & BATADV_WARNING_DEFAULT)) {
-		batadv_info(soft_iface,
+		batadv_info(hard_iface->soft_iface,
 			    "WiFi driver or ethtool info does not provide information about link speeds on interface %s, therefore defaulting to hardcoded throughput values of %u.%1u Mbps. Consider overriding the throughput manually or checking your driver.\n",
 			    hard_iface->net_dev->name,
 			    BATADV_THROUGHPUT_DEFAULT_VALUE / 10,
@@ -180,26 +151,31 @@ default_throughput:
 	}
 
 	/* if none of the above cases apply, return the base_throughput */
-	*pthroughput = BATADV_THROUGHPUT_DEFAULT_VALUE;
-	return true;
+	return BATADV_THROUGHPUT_DEFAULT_VALUE;
 }
 
 /**
  * batadv_v_elp_throughput_metric_update() - worker updating the throughput
  *  metric of a single hop neighbour
- * @neigh: the neighbour to probe
+ * @work: the work queue item
  */
-static void
-batadv_v_elp_throughput_metric_update(struct batadv_hardif_neigh_node *neigh)
+void batadv_v_elp_throughput_metric_update(struct work_struct *work)
 {
-	u32 throughput;
-	bool valid;
+	struct batadv_hardif_neigh_node_bat_v *neigh_bat_v;
+	struct batadv_hardif_neigh_node *neigh;
 
-	valid = batadv_v_elp_get_throughput(neigh, &throughput);
-	if (!valid)
-		return;
+	neigh_bat_v = container_of(work, struct batadv_hardif_neigh_node_bat_v,
+				   metric_work);
+	neigh = container_of(neigh_bat_v, struct batadv_hardif_neigh_node,
+			     bat_v);
 
-	ewma_throughput_add(&neigh->bat_v.throughput, throughput);
+	ewma_throughput_add(&neigh->bat_v.throughput,
+			    batadv_v_elp_get_throughput(neigh));
+
+	/* decrement refcounter to balance increment performed before scheduling
+	 * this task
+	 */
+	batadv_hardif_neigh_put(neigh);
 }
 
 /**
@@ -208,8 +184,8 @@ batadv_v_elp_throughput_metric_update(struct batadv_hardif_neigh_node *neigh)
  *
  * Sends a predefined number of unicast wifi packets to a given neighbour in
  * order to trigger the throughput estimation on this link by the RC algorithm.
- * Packets are sent only if there there is not enough payload unicast traffic
- * towards this neighbour..
+ * Packets are sent only if there is not enough payload unicast traffic towards
+ * this neighbour..
  *
  * Return: True on success and false in case of error during skb preparation.
  */
@@ -269,20 +245,18 @@ batadv_v_elp_wifi_neigh_probe(struct batadv_hardif_neigh_node *neigh)
  * batadv_v_elp_periodic_work() - ELP periodic task per interface
  * @work: work queue item
  *
- * Emits broadcast ELP message in regular intervals.
+ * Emits broadcast ELP messages in regular intervals.
  */
 static void batadv_v_elp_periodic_work(struct work_struct *work)
 {
-	struct batadv_v_metric_queue_entry *metric_entry;
-	struct batadv_v_metric_queue_entry *metric_safe;
 	struct batadv_hardif_neigh_node *hardif_neigh;
 	struct batadv_hard_iface *hard_iface;
 	struct batadv_hard_iface_bat_v *bat_v;
 	struct batadv_elp_packet *elp_packet;
-	struct list_head metric_queue;
 	struct batadv_priv *bat_priv;
 	struct sk_buff *skb;
 	u32 elp_interval;
+	bool ret;
 
 	bat_v = container_of(work, struct batadv_hard_iface_bat_v, elp_wq.work);
 	hard_iface = container_of(bat_v, struct batadv_hard_iface, bat_v);
@@ -318,8 +292,6 @@ static void batadv_v_elp_periodic_work(struct work_struct *work)
 
 	atomic_inc(&hard_iface->bat_v.elp_seqno);
 
-	INIT_LIST_HEAD(&metric_queue);
-
 	/* The throughput metric is updated on each sent packet. This way, if a
 	 * node is dead and no longer sends packets, batman-adv is still able to
 	 * react timely to its death.
@@ -344,28 +316,16 @@ static void batadv_v_elp_periodic_work(struct work_struct *work)
 
 		/* Reading the estimated throughput from cfg80211 is a task that
 		 * may sleep and that is not allowed in an rcu protected
-		 * context. Therefore add it to metric_queue and process it
-		 * outside rcu protected context.
+		 * context. Therefore schedule a task for that.
 		 */
-		metric_entry = kzalloc(sizeof(*metric_entry), GFP_ATOMIC);
-		if (!metric_entry) {
-			batadv_hardif_neigh_put(hardif_neigh);
-			continue;
-		}
+		ret = queue_work(batadv_event_workqueue,
+				 &hardif_neigh->bat_v.metric_work);
 
-		metric_entry->hardif_neigh = hardif_neigh;
-		list_add(&metric_entry->list, &metric_queue);
+		if (!ret)
+			batadv_hardif_neigh_put(hardif_neigh);
 	}
 	rcu_read_unlock();
 
-	list_for_each_entry_safe(metric_entry, metric_safe, &metric_queue, list) {
-		batadv_v_elp_throughput_metric_update(metric_entry->hardif_neigh);
-
-		batadv_hardif_neigh_put(metric_entry->hardif_neigh);
-		list_del(&metric_entry->list);
-		kfree(metric_entry);
-	}
-
 restart_timer:
 	batadv_v_elp_start_timer(hard_iface);
 out:
@@ -540,7 +500,7 @@ orig_free:
  * @skb: the received packet
  * @if_incoming: the interface this packet was received through
  *
- * Return: NET_RX_SUCCESS and consumes the skb if the packet was peoperly
+ * Return: NET_RX_SUCCESS and consumes the skb if the packet was properly
  * processed or NET_RX_DROP in case of failure.
  */
 int batadv_v_elp_packet_recv(struct sk_buff *skb,
